<?php
require_once(DIR_SYSTEM . 'library/phonepe/TransactionUtils.php');
require_once(DIR_SYSTEM . 'library/phonepe/ChecksumUtils.php');

class ControllerExtensionPaymentPhonepe extends Controller {

    public function index() {

        $this->load->language('extension/payment/phonepe');
        $this->load->model('extension/payment/phonepe');
        $this->load->model('checkout/order');

        $order_info = $this->model_checkout_order->getOrder($this->session->data['order_id']);

        $order_id = PhonepeHelper::getPhonepeOrderId($order_info['order_id']);
        $this->session->data['txn_id'] = $order_id;

        $cust_id = $email = $mobile_no = "";
        if(isset($order_info['telephone'])){
            $mobile_no = preg_replace('/\D/', '', $order_info['telephone']);
        }

        if(!empty($order_info['email'])){
            $cust_id = $email = trim($order_info['email']);
        } else if(!empty($order_info['customer_id'])){
            $cust_id = $order_info['customer_id'];
        }else{
            $cust_id = "CUST_".$order_id;
        }

        $amount = $this->currency->format($order_info['total'], $order_info['currency_code'], $order_info['currency_value'], false);
        $amountinpaisa = $amount * 100 ;

        $parameters = array(
            "merchantId" 		=> $this->config->get('payment_phonepe_merchant_id'),
            "transactionId" 	=> (string)$order_id,
            "merchantUserId" 	=> $cust_id,
            "amount" 		    => $amountinpaisa,
            "mobileNumber" 		=> $mobile_no,
            "email" 			=> $email
        );

        $myJSON = json_encode($parameters);
        $encodedPayload = base64_encode($myJSON);
        $reqParams = array("request" => $encodedPayload);

        $xverify      = PhonepeChecksum::phonepeCalculateChecksum($encodedPayload, $this->config->get('payment_phonepe_merchant_key'),$this->config->get('payment_phonepe_key_index'));

        $xcallbackurl = "";
        $xredirecturl =	$this->url->link('extension/payment/phonepe/callback');
        $txn_url      = phonepeHelper::getTransactionURL($this->config->get('payment_phonepe_environment'));

        //Accept Payment API
        $resParams = PhonepeHelper::executeCurlPhonePe($txn_url,$parameters,$xverify,$xcallbackurl,$xredirecturl);

        if($resParams['success']==false){
            echo $resParams;
            if(!empty($resParams)){
                $this->session->data['error'] = 'Transaction could not be initiated because of '.$resParams['code']. '. Please try again.';
            }
            else{
                $this->session->data['error'] = 'Transaction could not be initiated because of Network issue. Please check network connectivity.';
            }

            $data['action'] = $this->url->link('checkout/cart');
            $data['button_confirm']	= $this->language->get('button_confirm');

            if(file_exists(DIR_TEMPLATE . $this->config->get('config_template') . '/template/extension/payment/phonepe'))
            {
                return $this->load->view($this->config->get('config_template') . '/template/extension/payment/phonepe', $data);
            }
            else
            {
                return $this->load->view('extension/payment/phonepe', $data);
            }

        }
        else
        {
            $redirectURL = $resParams['data']['redirectURL'];
            $data['button_confirm']	= $this->language->get('button_confirm');
            $data['xverify']		    = $xverify;
            $data['xcallbackurl']		= $xcallbackurl;
            $data['xredirecturl']		= $xredirecturl;
            $data['fpfields']		    = $parameters;
            $data['txn_url']		    = $txn_url;
            $data['action']            = $redirectURL;


            if(file_exists(DIR_TEMPLATE . $this->config->get('config_template') . '/template/extension/payment/phonepe'))
            {
                return $this->load->view($this->config->get('config_template') . '/template/extension/payment/phonepe', $data);
            }
            else
            {
                return $this->load->view('extension/payment/phonepe', $data);
            }
        }

    }


    /**
     * phonepe sends response to callback
     */
    public function callback(){
        // load language and model
        $this->load->model('extension/payment/phonepe');
        $this->load->language('extension/payment/phonepe');
        $this->load->model('checkout/order');

        $data['title'] 			= sprintf($this->language->get('heading_title'), $this->config->get('config_name'));
        $data['heading_title'] 	= sprintf($this->language->get('heading_title'), $this->config->get('config_name'));
        $data['direction'] 		= $this->language->get('direction');
        $data['language'] 		= $this->language->get('code');

        $order_info  = $this->model_checkout_order->getOrder($this->session->data['order_id']);
        $order_id    = $this->session->data['txn_id'];

        $merchantId  = $this->config->get('payment_phonepe_merchant_id');
        $txnid       = (string)$order_id;
        $saltkey     = $this->config->get('payment_phonepe_merchant_key');
        $saltindex   = $this->config->get('payment_phonepe_key_index');

        $xverify     = PhonepeChecksum::phonepeStatusCalculateChecksum($merchantId, $txnid, $saltkey, $saltindex);
        $txn_url     = phonepeHelper::getTransactionStatusURL($this->config->get('payment_phonepe_environment'));

        $amount = $this->currency->format($order_info['total'], $order_info['currency_code'], $order_info['currency_value'], false);
        $amountinpaisa = $amount * 100 ;
        $amount_returned = 0;
        $mycounter=0;
        $dataMsg = '';

        do{
            $resStatus= PhonepeHelper::executeCurlPhonePeStatus($txn_url,$xverify,$merchantId,$txnid );
            $mycounter++ ;
            sleep(2);
        } while(($resStatus['code']==PhonepeConstants::PENDING && $mycounter < PhonepeConstants::MAX_RETRY_COUNT) || ($resStatus['code']==PhonepeConstants::SERVER_ERROR && $mycounter < PhonepeConstants::MAX_RETRY_COUNT));



        if($resStatus['code']==PhonepeConstants::TXN_NOT_FOUND){
            $data['text_response']  = sprintf($this->language->get('text_response'),$resStatus['message'] );
            $data['payment_status'] = $resStatus['code'];
        }
        else
        {
            $data['text_response'] 	= sprintf($this->language->get('text_response'),$resStatus['message'] );
            $data['payment_status'] = $resStatus['code'];
            $amount_returned = $resStatus['data']['amount'];

            /* save phonepe response in db */
            if(PhonepeConstants::SAVE_PHONEPE_RESPONSE && !empty($resStatus['code'])){
                $this->model_extension_payment_phonepe->saveTxnResponse($resStatus, PhonepeHelper::getOrderId($resStatus['data']['transactionId']));
            }

        }

        if($resStatus['code'] == PhonepeConstants::SUCCESS && $amountinpaisa==$amount_returned){

            $comment = sprintf($this->language->get('text_transaction_id'), $txnid) .'<br/>'. sprintf($this->language->get('text_phonepe_order_id'), $order_id);
            $statusId = $this->config->get('payment_phonepe_order_success_status_id');
        }

        else if($resStatus['code'] == PhonepeConstants::PENDING){
            $comment = '' ;
            $statusId = $this->config->get('payment_phonepe_order_pending_status_id');
            $dataMsg = $this->language->get('text_pending');
        }

        else {
            $comment = '';
            $statusId = $this->config->get('payment_phonepe_order_failed_status_id');
            $dataMsg = $this->language->get('text_failure');
        }

        $this->addOrderHistory($order_id, $this->config->get('payment_phonepe_order_success_status_id'), $comment);
        $this->session->data['error'] = $dataMsg;
        $this->preRedirect($data);
    }


    private function addOrderHistory($order_id, $order_status_id, $comment = ''){
        try{
            $this->model_checkout_order->addOrderHistory($order_id, $order_status_id, $comment);
        } catch(\Exception $e){
        }
    }

    /**
     * show template while response
     */
    private function preRedirect($data){

        $data['continue'] = $this->url->link('checkout/cart');

        if(!empty($data['payment_status'])){
            if($data['payment_status'] == PhonepeConstants::SUCCESS){
                $data['continue'] 			= $this->url->link('checkout/success');
                $data['text_message'] 		= $this->language->get('text_success');
                $data['text_message_wait'] 	= sprintf($this->language->get('text_success_wait'), $this->url->link('checkout/success'));
            }else if($data['payment_status'] == PhonepeConstants::PENDING){
                $data['text_message'] 		= $this->language->get('text_pending');
                $data['text_message_wait'] 	= sprintf($this->language->get('text_pending_wait'), $this->url->link('checkout/cart'));
            }else{
                $data['text_message'] 		= $this->language->get('text_failure');
                $data['text_message_wait'] 	= sprintf($this->language->get('text_failure_wait'), $this->url->link('checkout/cart'));
            }

        }

        $this->response->setOutput($this->load->view('extension/payment/phonepe_response', $data));
    }

    /**
     * check cURL working or able to communicate with phonepe
     */
    public function curlTest(){

        $debug = array();

        if(!function_exists("curl_init")){
            $debug[0]["info"][] = "cURL extension is either not available or disabled. Check phpinfo for more info.";

            // if curl is enable then see if outgoing URLs are blocked or not
        } else {

            // if any specific URL passed to test for
            if(!empty($this->request->get["url"])){
                $testing_urls = array(urldecode($this->request->get["url"]));
            } else {

                // this site homepage URL
                $server = (!empty($this->request->server['HTTPS'])? HTTPS_SERVER : HTTP_SERVER);

                $testing_urls = array(
                    $server,
                    "https://www.gstatic.com/generate_204",
                    PhonepeConstants::TRANSACTION_STATUS_URL_PRODUCTION,
                    PhonepeConstants::TRANSACTION_STATUS_URL_STAGING
                );
            }

            // loop over all URLs, maintain debug log for each response received
            foreach($testing_urls as $key => $url){

                $debug[$key]["info"][] = "Connecting to <b>" . $url . "</b> using cURL";

                $ch = curl_init($url);
                curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);

                $res = curl_exec($ch);
                $http_code = curl_getinfo($ch, CURLINFO_HTTP_CODE);
                if (!curl_errno($ch)) {
                    $info1 = "cURL executed succcessfully.";
                    $info2 = "HTTP Response Code: <b>". $http_code . "</b>";
                    $info3 = "";
                } else {
                    $info1 = "Connection Failed !!";
                    $info2 = "Error Code: <b>" . curl_errno($ch) . "</b>";
                    $info3 = "Error: <b>" . curl_error($ch) . "</b>";
                }

                $debug[$key]["info"][] = $info1;
                $debug[$key]["info"][] = $info2;
                $debug[$key]["info"][] = $info3;

                if((!empty($this->request->get["url"])) || (in_array($url, array(PhonepeConstants::TRANSACTION_STATUS_URL_PRODUCTION , PhonepeConstants::TRANSACTION_STATUS_URL_STAGING)))){
                    $debug[$key]["info"][] = "Response: <br/><!----- Response Below ----->" . $res;
                }

                curl_close($ch);
            }
        }

        foreach($debug as $k => $v){
            echo "<ul>";
            foreach($v["info"] as $info){
                echo "<li>". $info ."</li>";
            }
            echo "</ul>";
            echo "<hr/>";
        }
    }
}
?>